function [ innerCVscheme ] = CreateInnerTestSets( n , m , outerCVscheme , kfold)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CreateInnerTestSets constructs inner test sets for k-fold random cross
% validation to determine the optimal value of lambda and the number of
% Principal Components for different datasets with different outer test
% sets.
%
% INPUT:
% n:       Number of rows (time points) in the concentration data matrix
% m:       Number of columns (metabolites) in the concentration data matrix
% outerCVscheme:     Matrix with 'kfold' rows. Each row of the matrix
%                    stores a single test set for a single run. The whole 
%                    matrix contains all the test set points that have to
%                    be used.
% kfold:   Scalar indicating how many partitions the whole dataset will be 
%          divided into to perform Cross Validation.
%
% OUTPUT:
% innerCVscheme:      Cell Array with the number of cells equal to the
% number of different outer test sets. Each cell contains a different 3-D
% inner test set matrix whose each row stores a single test set for a
% single run. In the 3rd dimension, the matrix stores CV schemes obtained 
% by 'kfold' number of different permutations of the indices. 
%
% Programmed by Dicle Hasdemir
% @ Biosystem Data Analysis group, Swammerdam Institute for Life Sciences,
% University of Amsterdam.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
innerCVscheme = {};

% Find the indices for the first and last time points
flp = zeros ( 2 * m , 1 );
for j = 1 : m
    flp ( (j-1) * 2 + 1) = (j-1) * n + 1;
    flp ( (j-1) * 2 + 2) = (j-1) * n + n; 
end

% Remove the first and the last time points (flp)
temp = setdiff ( 1 : n * m , flp );

% Remove the outer test set points
for i = 1 : size ( outerCVscheme,1 )
    testpool = setdiff ( temp , outerCVscheme ( i,: ) );
    NpInSet = length ( testpool ) / kfold;
    Tsetperms = zeros ( kfold , NpInSet , kfold );
    for k = 1 : kfold
        PermInd = randperm ( length ( testpool ) );
        PermTest = testpool ( PermInd );   
        TSet = reshape ( PermTest , kfold , NpInSet );
        Tsetperms ( : , : , k ) = TSet;
    end
    innerCVscheme {i} = Tsetperms;
end
save innerCVscheme innerCVscheme
