function [ press_FINAL , exitflags , OptDiagnosticsTable ] = OptimizeLambdaAndPc ( cmpdataset , TimePoints , weightmatrix , outerCVscheme , innerCVscheme , lambdas , pcs , OutputFileName , plotoption)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OptimizeLambdaAndPc optimizes the number of PC s and Lambda for the SPCA.
%
% INPUT:
% cmpdataset        Concentration Data Matrix
% TimePoints        Measurement Time Points
% weightmatrix      W matrix for the calculations without any zeros. It
%                   contains only 1's if no specific weighting is intended.
% outerCVscheme     Matrix with 'kfold' rows. Each row of the matrix
%                   stores a single test set for a single run. The whole 
%                   matrix contains all the test set points that have to
%                   be used.
% innerCVscheme     Cell Array with the number of cells equal to the
%                   number of different outer test sets. Each cell 
%                   contains a different 3-D inner test set matrix whose 
%                   each row stores a single test set for a single run. 
%                   In the 3rd dimension, the matrix stores CV schemes 
%                   obtained by 'kfold' number of different permutations 
%                   of the indices. 
% lambdas           Row vector containing the lambdas to be tested
% pcs               Row vector containing the PCs to be tested
% OutputFileName    The name of the file where the results will be
%                   exported.
% plotoption        0, 1 or 2. 0 gives no figure output during the
%                   optimizations performed. 1 plots the change in the 
%                   normalized residuals during the optimization. 2 plots 
%                   the change in the values of the variables. 
%
% OUTPUT:
% press_FINAL       A 3-D matrix with final Prediction Error Values, with
%                   changing value of lambda in the rows and PCs in the 
%                   columns and different outer test sets in the 3rd 
%                   dimension. 
% exitflags         Vector containing all the exitflags from all the
%                   optimization runs performed. It can be used for 
%                   diagnostics purposes. See the help file for 'lsqnonlin'
%                   for details.
% OptDiagnosticsTable   Contains the output messages from the optimization
%                       runs performed for diagnostics purposes.
%
% Programmed by Dicle Hasdemir
% @ Biosystem Data Analysis group, Swammerdam Institute for Life Sciences,
% University of Amsterdam. Last Modification Date: September 2013.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% size of the dataset
[ n , m ] = size ( cmpdataset );

% initialize outputs
OptDiagnosticsTable = {};
exitflags = [];
press_FINAL = zeros ( length ( lambdas ) , length ( pcs ) , size ( outerCVscheme , 1 ) );

% prepare the output file
fileID = fopen(OutputFileName,'w');
fprintf ( fileID , '%s\n' , 'Lambdas Covered (in Rows)' );
fprintf ( fileID , '%15.8f\n' , lambdas );
fprintf ( fileID , '%s\n' , 'PCs Covered (in Columns)' );
fprintf ( fileID , '%i\n' , pcs);

%tic
for kk = 1 : size ( outerCVscheme , 1 )
    
    % outer test indices for this run
    outertest_ind = outerCVscheme ( kk , : );
    
    % all inner test sets for this run
    Innertestset = innerCVscheme { kk };
    kfold = size ( Innertestset , 1 );
    
    for ii = 1 : length ( lambdas )
        
        for jj = 1 : length ( pcs )
            
            fprintf(1,['\n PC = ' int2str(pcs(jj)) ' & Lambda = ' num2str(lambdas(ii)) ' for the dataset with the outer test set "' int2str(kk) '" is being processed ...\n\n']);
            
            temp2 = 0;
            
            for nn = 1 : kfold
                
                temp = 0;
                
                for ll = 1 : kfold
                    
                    % restore the original matrices for this run
                    Wtest = weightmatrix;
                    Wtr = weightmatrix;
                    msdataset = cmpdataset;
                    
                    % inner test set for this run
                    innertest_ind = Innertestset ( ll , : , nn );
                    
                    % complete test set for this run
                    test_ind = [ outertest_ind ,  innertest_ind ];
                    train_ind = setdiff (  1 : n * m  , innertest_ind  );
                    
                    % calculate starting points for Z and P using PCA
                    msdataset ( test_ind ) = 0;
                    [ u0 , s0 , v0 ] = svd ( msdataset , 0 );
                    Zini = u0 ( : , 1 : pcs ( jj ) ) * s0 ( 1 : pcs ( jj ) , 1 : pcs ( jj ) );
                    Pini = v0 ( : , 1 : pcs ( jj ) );
                    
                    % call nonlinear smooth pca function
                    Wtr ( test_ind )  = 0;
                    [ exitflag , output , Zsol , Psol ] = nlspcaw ( cmpdataset , Wtr , pcs (jj) , lambdas (ii) , TimePoints , Zini , Pini , plotoption);
                    OptDiagnosticsTable { end + 1 } = { [ 'Lambda = ' num2str(lambdas(ii)) ', PC = ' int2str(pcs(jj)) ', Dataset with outer test set no: ' int2str(kk)] , output.message };
                    exitflags = [ exitflags ; exitflag ];
                    
                    % Prediction of Training and Test Sets
                    cmpBack = Zsol * Psol';
                    
                    % Calculate Prediction Residuals for a single Test Set
                    Wtest ( train_ind ) = 0;
                    resmat = Wtest .* ( cmpdataset - cmpBack);
                    resmat = resmat (:);
                    resi = sum ( resmat.^2 );
                    
                    % Calculate Prediction Residuals for all Test Set
                    % Elements
                    temp = temp + resi;
                end
                
                
                temp2 = temp2 + temp;
            end
            % Calculate the average Prediction Error from kfold CV
            allpress = temp2 / kfold;
            
            press_FINAL(ii,jj,kk) = allpress;
        end
    end
    
    % Print Results to output file
    fprintf ( fileID , '%s\n' , ['press_FINAL for the dataset with outer test set: ' int2str((kk)) ] );
    
    printformat = [];
    if length ( pcs ) == 1;
        printformat = '%15.4f\n';
    else
        for ii = 1 : length ( pcs ) - 1
            printformat = [ printformat '%15.4f\t' ];
        end
        printformat = [ printformat '%15.4f\n' ];
    end
    for ii = 1 : length ( lambdas )
        fprintf ( fileID , printformat , press_FINAL(ii,:,kk) ) ;
    end
end
%toc
fclose(fileID);