function [testres,exitflags] = InvalidationMain ( X , TI , outerCVscheme , stpoints , weightmatrix , lambdas , pcs )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% InvalidationMain performs SPCA and ODE model based prediction on the
% dataset using the outer cross validation scheme (diagonal stratified CV 
% as defined in the article) .
%
% INPUT: 
% X             Concentration Data 
% TI            Measurement Time Points Vector 
% stpoints      Starting point of the variables for optimization. It
%               should be supplied based on biological prior knowledge. The
%               last m (number of metabolites) values are initial estimates
%               for the concentration values at the initial measurement
%               point (First element of the TI vector). The first values 
%               are initial estimates for the rate parameters. 
% weightmatrix  W matrix for the calculations without any zeros. It
%               contains only 1's if no specific weighting is intended.
% lambdas       Lambda values vector for all different outer test sets. If
%               the whole dataset is divided to 'k' partitions for CV, this 
%               vector has to contain 'k' elements. 
% pcs           PC values vector for all different outer test sets. If
%               the whole dataset is divided to 'k' partitions for CV, this 
%               vector has to contain 'k' elements. 
%
% OUTPUT:
% testres       Vector containing the prediction residuals for each
%               different outer test set in the rows and two different 
%               methods (SPCA and ODE, respectively) in the columns.
% exitflags     Vector containing all the exitflags from all the
%               optimization runs performed. It can be used for 
%               diagnostics purposes. See the help file for 'lsqnonlin'
%               for details.
%
% Programmed by Dicle Hasdemir
% @ Biosystem Data Analysis group, Swammerdam Institute for Life Sciences,
% University of Amsterdam. Last Modification Date: September 2013.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

plotoption = 0;
testsetno = size ( outerCVscheme , 1 );

testres = zeros ( testsetno , 2 );
exitflags = zeros ( testsetno , 2 );

for testsetcount = 1 : testsetno
    fprintf(1,['\n Dataset with the outer test set "' int2str(testsetcount) '" is being processed ...\n\n']);

    pc = pcs ( testsetcount );
    lambda = lambdas ( testsetcount );
    
    %%% Each row of the outerCVscheme matrix gives the test indices for one
    %%% test set
    test_indices = ( outerCVscheme ( testsetcount , : ) )';
    training_indices = setdiff ( 1 : size ( X , 1 ) * size ( X , 2 ) , test_indices )';
    
    %%% Create Wtr with zeros on the test set points
    Wtr = weightmatrix;
    Wtr ( test_indices ) = 0;
    
    %%% Create Wtest with zeros on the training set points
    Wtest = weightmatrix;
    Wtest ( training_indices ) = 0;
    
    %%%%%%%%%%%%%%%%%%%%%%%% SMOOTH PCA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Starting points Zini (Scores) and Pini (Loadings) for the
    %%% optimization. Test set points are imputed by interpolating the
    %%% adjacent points and imputed X is stored in Xinitial. A PCA 
    %%% decomposition on Xinitial gives Zini and Pini.
    Xinitial = X;
    Xinitial ( test_indices ) = ( Xinitial ( test_indices - 1 ) + Xinitial ( test_indices + 1 ) ) / 2;
    [ u0 , s0 , v0 ] = svd ( Xinitial , 0 );
    Zini = u0 ( : , 1 : pc ) * s0 ( 1 : pc , 1 : pc );
    Pini = v0 ( : , 1 : pc );
    
    % SMOOTH pca on the data with missing values
    [ exitflag , ~ , Zsol , Psol ] = nlspcaw( X , Wtr , pc , lambda , TI , Zini , Pini , plotoption );
    exitflags ( testsetcount , 1 ) = exitflag;
    Xback_spca = Zsol * Psol';
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%% KINETIC MODELING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% Kinetic part optimization
    numrpar = length ( stpoints ) - size ( X , 2 );
    lb = 0 * ones ( size ( stpoints ) );
    ub = Inf * ones ( size ( stpoints ) );
    options = optimset ( 'Display' , 'off' );
    [ x , ~ , ~ , flag ] = lsqnonlin ( @(x)dif_exp_pred(x, X, TI, Wtr, numrpar) , stpoints , lb , ub , options );
    exitflags ( testsetcount , 2 ) = flag;
    %%% Simulate the concentration curves with the optimal parameters
    [~ , Xback_ode ] = dif_exp_pred(x, X, TI, Wtr, numrpar);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%% PREDICTION ERROR %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    WeighRes_spca = Wtest .* ( X - Xback_spca );
    WeighRes_ode = Wtest .* ( X - Xback_ode );
    testres ( testsetcount , 1 ) = sum( WeighRes_spca ( : ) .^2 );
    testres ( testsetcount , 2 ) = sum( WeighRes_ode ( : ) .^2 );
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    %%%%%%%%%%%%%%%%%%%%%%%% SHOW PREDICTIONS ON THE GRAPH %%%%%%%%%%%%%%%%
    [ testx , testy ] = ind2sub ([size( X , 1 ) , size( X , 2 )] , test_indices );
    for ii = 1 : size ( X , 2 )
        figure ( ii )
        plot ( TI , X ( : , ii ) , 'b*')
        hold all
        cvind = find ( testy == ii );
        plot (TI(testx(cvind)),Xback_spca(testx(cvind),testy(cvind)),'o','MarkerFaceColor',[0.48 0.06 0.89],'MarkerEdgeColor',[0.48 0.06 0.89],'MarkerSize',6)
        plot (TI,Xback_ode(:,ii),'r');
        title (['Species' int2str(ii)])
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end

TPE = sum ( testres );
fprintf (1,'%%%% Total Prediction Error %%%%\n');
fprintf (1,['SPCA: ' num2str(TPE(1)) '\n']);
fprintf (1,['ODE: ' num2str(TPE(2)) '\n']);

if TPE(1) < TPE(2)
    fprintf (1,'The model was invalidated\n');
else
    fprintf (1,'The model could not be invalidated\n');
end

