function [ exitflag , output , Z , P ] = nlspcaw ( X , W , nc , lambda , time , Z0 , P0 , plotoption)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NLSPCAW  Weighted Smooth PCA by using non-linear least-squares; finds Z, P
%from Min||W.*(X-ZP')||^2+lambda*||D*Z||^2 with normalized loadings P
%
% INPUT:
% X             data [NxM]
% W             weighting matrix with zeros for the test set elements
% nc            scalar indicating the number of principal components
% lambda        scalar indicating the smoothing parameter Lambda
% time          column vector with measurement time points
% Z0            Starting matrix for the Smooth Scores
% P0            Starting matrix for the Loadings
% plotoption    0, 1 or 2. 0 gives no figure output during the
%               optimizations performed. 1 plots the change in the 
%               normalized residuals during the optimization. 2 plots 
%               the change in the values of the variables. 
%
% OUTPUT:
% Z             Smooth Scores [Nxnc]
% P             Loadings [Mxnc]
% exitflag      Exitflag from the optimization. It can be used for 
%               diagnostics purposes. See the help file for 'lsqnonlin'
%               for details.
% output        Output Message from the optimization. It can be used for 
%               diagnostics purposes. See the help file for 'lsqnonlin'
%               for details.
%
% Requirements
% ------------
% Optimization Toolbox

% Programmed by Maikel Verouden and modified by Dicle Hasdemir
% @ Biosystem Data Analysis group,Swammerdam Institute for Life Sciences,
% Universiteit van Amsterdam
% Last Modification Date: September 2013.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin < 8
    error('Error: Check the number of inputs.')
end

%%% Determine dimensions of X
[N,M] = size(X);

%%% Create the second order derivative matrix D
D = zeros(N-2,N);
for i = 2:N-1
    h = time(i) - time(i-1);
    k = time(i+1) - time(i);
    D(i-1,i-1) = 2*h / (k*h*h + h*k*k);
    D(i-1,i) = -2*(k+h) / (k*h*h + h*k*k);
    D(i-1,i+1) = 2*k / (k*h*h + h*k*k);
end

%%% Options for the lsqnonlin function
switch plotoption
    case 0
        options = optimset(optimset('lsqnonlin'),'Display','off','MaxFunEvals',1e20,'MaxIter',1e3);
    case 1
        options = optimset(optimset('lsqnonlin'),'Display','off','MaxFunEvals',1e20,'PlotFcns',@optimplotresnorm,'MaxIter',1e3);
    case 2
        options = optimset(optimset('lsqnonlin'),'Display','off','MaxFunEvals',1e20,'PlotFcns',@optimplotx,'MaxIter',1e3);
end

%%% Starting vector for the optimization
invec0 = [Z0(:);P0(:)];

%%% Optimization run
[outvec,~,~,exitflag,output] = lsqnonlin(@SPCAWfun,invec0,[],[],options,N,M,X,W,D,lambda,nc);

%%% Recreate Z and P from the results
Z = reshape(outvec(1:(N*nc)),N,nc);
P = reshape(outvec((N*nc+1):(N*nc+M*nc)),M,nc);

%%% normalize the final P and correct Z accordingly
sP=zeros(nc,1);
for r=1:nc
    sP(r) = sqrt(P(:,r)'*P(:,r));
    P(:,r) = P(:,r)/sP(r);
    Z(:,r) = Z(:,r)*sP(r);
end
end

%%% Objective function to be minimized
function [outfun]=SPCAWfun(invec,N,M,X,W,D,lambda,nc)
%%% reform the invec to scores and loadings
Z = reshape(invec(1:(N*nc)),N,nc);
P = reshape(invec((N*nc+1):(N*nc+M*nc)),M,nc);

%%% normalize each column of PP and addjust ZZ accordingly
sPP=zeros(nc,1);
for r=1:nc
    sPP(r) = sqrt(P(:,r)'*P(:,r));
    P(:,r) = P(:,r)/sPP(r);
    Z(:,r) = Z(:,r)*sPP(r);
end
EE = W.*(X-Z*P');
lDZ = sqrt(lambda)*D*Z;
outfun = [EE(:);lDZ(:)];
end
